<?php
namespace Ep\Gath;

use Ep\Core\UserProviderInterface;
use Ep\Gath\Adapter\AdapterInterface;
use Ep\Gath\Entity\Price;
use Ep\Gath\Entity\Product;
use Ep\Gath\Entity\Stock;
use Zend\Hydrator\HydratorInterface;
use Zend\ServiceManager\PluginManagerInterface;

/**
 * Integracija s Gathom.
 *
 * @package Ep\Service
 * @author Elektronika Perhoč <support@ep.hr>
 * @link https://bitbucket.org/elektronika-perhoc/ep-webshop
 */
class GathService
{
	const PRODUCT_ID_OFFSET = 1000000;

	// Po vremenu unosa silazno
	const SORT_DATE_DESC = 1;
	// Po cijeni uzlazno
	const SORT_PRICE_ASC = 2;
	// Po cijeni silazno
	const SORT_PRICE_DESC = 3;

	/** @var AdapterInterface */
	private $adapter;
	/** @var UserProviderInterface */
	private $users;
	/** @var PluginManagerInterface */
	private $hydrators;
	/** @var int */
	private $foundRows;

	/**
	 * @return int
	 */
	public function getFoundRows()
	{
		return intval($this->foundRows);
	}

	/**
	 * GathService constructor.
	 *
	 * @param AdapterInterface $adapter
	 * @param UserProviderInterface $users
	 * @param PluginManagerInterface $hydrators
	 */
	public function __construct(
		AdapterInterface $adapter,
		UserProviderInterface $users,
		PluginManagerInterface $hydrators
	) {
		$this->adapter = $adapter;
		$this->users = $users;
		$this->hydrators = $hydrators;
	}

    /**
     * Vraća proizvode za datu kategoriju.
     *
     * @param int $categoryId
     * @param int $order
     * @param int $limit
     * @param int $offset
     *
     * @return Product[]|object[]
     */
	public function getProductsByCategory(int $categoryId, int $order = GathService::SORT_DATE_DESC, int $limit = 10, $offset = 0)
	{
		$data = $this->adapter->getProductsByCategory($categoryId, $order, $limit, $offset);

		if (method_exists($this->adapter, 'getFoundRows')) {
			$this->foundRows = $this->adapter->getFoundRows();
		} else {
			$this->foundRows = 0;
		}

		/** @var Product[] $entities */
		$entities = $this->hydrateEntities(Product::class, $data);
		return $entities;
	}

	/**
	 * @param array $productIds
	 *
	 * @return array
	 */
	public function getProductsByIds(array $productIds)
	{
		$productIds = array_filter($productIds, function($productId) {
			return $productId > 0;
		});

		if (0 === count($productIds)) {
			return [];
		}

		$data = $this->adapter->getProductsByIds($productIds);

		/** @var Product[] $entities */
		$entities = $this->hydrateEntities(Product::class, $data);
		$keyedEntities = [];

		foreach ($entities as $entity) {
			$keyedEntities[$entity->getId()] = $entity;
		}

		$sorted = [];
		foreach ($productIds as $productId) {
			if (isset($keyedEntities[$productId])) {
				$sorted[] = $keyedEntities[ $productId ];
			}
		}

		return $sorted;
	}

	public function getTecDocProducts(array $tecDocPairs, int $order, int $limit = 10, int $offset = 0)
	{
		$data = $this->adapter->getTecDocProducts($tecDocPairs, $order, $limit, $offset);

        if (method_exists($this->adapter, 'getFoundRows')) {
            $this->foundRows = $this->adapter->getFoundRows();
        } else {
            $this->foundRows = 0;
        }

		/** @var Product[] $entities */
		$entities = $this->hydrateEntities(Product::class, $data);
		return $entities;
	}

	public function getMotoProducts($motoId, $articleIds, $order, $limit, $offset)
    {
        $data = $this->adapter->getMotoProducts($motoId, $articleIds, $order, $limit, $offset);

        if (method_exists($this->adapter, 'getFoundRows')) {
            $this->foundRows = $this->adapter->getFoundRows();
        } else {
            $this->foundRows = 0;
        }

        /** @var Product[] $entities */
        $entities = $this->hydrateEntities(Product::class, $data);
        return $entities;
    }

	/**
	 * @param array $skus
	 *
	 * @return Stock[]|object[]
	 */
	public function getProductStock(array $skus)
	{
		$username = $this->users->hasUser() && $this->users->isWholesale() ? $this->users->getUsername() : null;
		$data = $this->adapter->getStock($skus, $username);

		/** @var Stock[] $entities */
		$entities = $this->hydrateEntities(Stock::class, $data);
		return $entities;
	}

	/**
	 * Vraća cijene i stanje zaliha za date proizvode.
	 *
	 * @param array $skus Skuovi proizvoda.
	 *
	 * @return Price[]
	 */
	public function getProductPrices(array $skus)
	{
		$webshopOptions = get_option('ep_webshop');

		$retail = isset($webshopOptions['type']) && $webshopOptions['type'] === 'retail';
		$wholesale = isset($webshopOptions['type']) && $webshopOptions['type'] === 'wholesale';
		$discounts = true;

		$username = $this->users->hasUser() && $this->users->isWholesale() ? $this->users->getUsername() : null;
		$data = $this->adapter->getPrices($skus, $retail, $wholesale, $discounts, $username);

		/** @var Price[] $entities */
		$entities = $this->hydrateEntities(Price::class, $data);

		return $entities;
	}

	/**
	 * Puni niz entiteta s podacima.
	 *
	 * @param string $entityClass
	 * @param array $data
	 *
	 * @return object[]
	 */
	private function hydrateEntities(string $entityClass, array $data)
	{
		$entities = array_map(function($data) use ($entityClass) {
			return $this->hydrateEntity($entityClass, $data);
		}, $data);

		return $entities;
	}

	/**
	 * Puni jedan entitet s podacima.
	 *
	 * @param string $entityClass
	 * @param array $data
	 *
	 * @return object
	 */
	private function hydrateEntity(string $entityClass, array $data)
	{
		$hydrator = $this->getHydrator($entityClass);
		return $hydrator->hydrate($data, new $entityClass());
	}

	/**
	 * Vraća hidrator po imenu.
	 *
	 * @param string $name
	 *
	 * @return HydratorInterface
	 */
	private function getHydrator(string $name)
	{
		return $this->hydrators->get($name);
	}
}